﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using System.Windows.Forms;

namespace Employees
{
    public partial class Employees : Form
    {
        private static DataTable dt;
        //private TextBox txtBox = new TextBox();
        private static int dataColumnsCount = 9;

        public Employees()
        {
            InitializeComponent();
            dt = new DataTable("Employees");
            createDataTable();
        }

        /// <summary>
        /// Creates the employee data table for the project.
        /// </summary>
        public void createDataTable()
        {
            DataColumn column;
            DataColumn[] keys = new DataColumn[1];

            column = new DataColumn();
            column.ColumnName = "id";
            column.DataType = System.Type.GetType("System.String");
            dt.Columns.Add(column);
            keys[0] = column;
            dt.PrimaryKey = keys;
            
            column = new DataColumn();
            column.ColumnName = "firstName";
            column.DataType = System.Type.GetType("System.String");
            dt.Columns.Add(column);

            column = new DataColumn();
            column.ColumnName = "lastName";
            column.DataType = System.Type.GetType("System.String");
            dt.Columns.Add(column);

            column = new DataColumn();
            column.ColumnName = "salary";
            column.DataType = System.Type.GetType("System.Int32");
            dt.Columns.Add(column);

            column = new DataColumn();
            column.ColumnName = "bonus";
            column.DataType = System.Type.GetType("System.Int32");
            dt.Columns.Add(column);

            column = new DataColumn();
            column.ColumnName = "department";
            column.DataType = System.Type.GetType("System.String");
            dt.Columns.Add(column);

            column = new DataColumn();
            column.ColumnName = "hireDate";
            column.DataType = System.Type.GetType("System.String");
            dt.Columns.Add(column);

            column = new DataColumn();
            column.ColumnName = "phoneNumber";
            column.DataType = System.Type.GetType("System.String");
            dt.Columns.Add(column);

            column = new DataColumn();
            column.ColumnName = "email";
            column.DataType = System.Type.GetType("System.String");
            dt.Columns.Add(column);

            FarPoint.Win.Spread.Data.SpreadDataBindingAdapter data = new FarPoint.Win.Spread.Data.SpreadDataBindingAdapter();
            data.DataSource = dt;
            data.Spread = fpSpread1;
            data.SheetName = "Employees Table";
            data.MapperInfo = new FarPoint.Win.Spread.Data.MapperInfo(2, 0, 0, 6);
            data.FillSpreadDataByDataSource();
        }

        /// <summary>
        /// Gets the data table of employees.
        /// </summary>
        /// <returns>the data table of employees</returns>
        public static DataTable getDataTable()
        {
            return dt;
        }

        /// <summary>
        /// Sets the employee data table to the specified data table.
        /// </summary>
        /// <param name="newDataTable">the data table to set the employee data table equal to</param>
        public static void setDataTable(DataTable newDataTable)
        {
            dt = newDataTable;
        }

        /// <summary>
        /// Adds an employee with the specified information to the data table.
        /// </summary>
        /// <param name="id">ID of the employee</param>
        /// <param name="firstName">first name of the employee</param>
        /// <param name="lastName">last name of the employee</param>
        /// <param name="salary">annual salary amount of the employee</param>
        /// <param name="bonus">bonus amount of the employee</param>
        /// <param name="department">department that the employee works in</param>
        /// <param name="hireDate">the date that the employee was hired</param>
        /// <param name="phoneNumber">the phone number of the employee</param>
        /// <param name="email">the email of the employee</param>
        public static void addToDataTable(string id, string firstName, string lastName, int salary, int bonus, string department, string hireDate, string phoneNumber, string email)
        {
            DataRow row = dt.NewRow();
            row["id"] = id;
            row["firstName"] = firstName;
            row["lastName"] = lastName;
            row["salary"] = salary;
            row["bonus"] = bonus;
            row["department"] = department;
            row["hireDate"] = hireDate;
            row["phoneNumber"] = phoneNumber;
            row["email"] = email;
            dt.Rows.Add(row);
        }

        /// <summary>
        /// Edits the employee with the specified information.
        /// </summary>
        /// <param name="id">ID of the employee</param>
        /// <param name="firstName">first name of the employee</param>
        /// <param name="lastName">last name of the employee</param>
        /// <param name="salary">annual salary amount of the employee</param>
        /// <param name="bonus">bonus amount of the employee</param>
        /// <param name="department">department that the employee works in</param>
        /// <param name="hireDate">the date that the employee was hired</param>
        /// <param name="phoneNumber">the phone number of the employee</param>
        /// <param name="email">the email of the employee</param>
        public static void editDataTable(string id, string firstName, string lastName, int salary, int bonus, string department, string hireDate, string phoneNumber, string email)
        {
            DataRow row = dt.Rows.Find(id);
            row["id"] = id;
            row["firstName"] = firstName;
            row["lastName"] = lastName;
            row["salary"] = salary;
            row["bonus"] = bonus;
            row["department"] = department;
            row["hireDate"] = hireDate;
            row["phoneNumber"] = phoneNumber;
            row["email"] = email;
        }

        /// <summary>
        /// Tests if the specified string is an integer.
        /// </summary>
        /// <param name="text">the string to test</param>
        /// <param name="property">the name of the property being tested</param>
        /// <returns>the boolean value of whether or not the string is an integer</returns>
        public static bool testIfNumber(string text, string property)
        {
            int num;
            if (!int.TryParse(text, out num))
            {
                DialogResult dr = MessageBox.Show(property + " must be a number.",
                                                    property,
                                                    MessageBoxButtons.OK, MessageBoxIcon.Warning);
                if (dr == DialogResult.OK)
                {
                    return false;
                }
            }
            return true;
        }

        /// <summary>
        /// Tests if an employee with the specified ID exists in the data table.
        /// </summary>
        /// <param name="id">the employee ID tolook for</param>
        /// <returns>the boolean value of whether or not an employee with the specified ID exists</returns>
        public static bool testIfExists(string id)
        {
            if (dt.Rows.Find(id) != null)
            {
                DialogResult dr = MessageBox.Show("ID already exists.", "Invalid ID",
                                                    MessageBoxButtons.OK, MessageBoxIcon.Warning);
                if (dr == DialogResult.OK)
                {
                    return false;
                }
            }
            return true;
        }

        /// <summary>
        /// Starts up the form for adding an employee to the spreadsheet and data table.
        /// </summary>
        /// <returns>the boolean value of whether or not the user choose to add the employee or not</returns>
        public bool addEmployeeForm()
        {
            AddEmployee newEmployee = new AddEmployee();
            newEmployee.ShowDialog();
            if (newEmployee.DialogResult == System.Windows.Forms.DialogResult.Cancel)
                return false;
            return true;
        }

        /// <summary>
        /// Opens up the form for searching for an employee in the data table.
        /// </summary>
        public static bool searchEmployeeDialog()
        {
            SearchForm search = new SearchForm();
            search.ShowDialog();
            if (search.DialogResult == System.Windows.Forms.DialogResult.Cancel)
                return false;
            return true;
        }

        /// <summary>
        /// Opens up the form for viewing and editting details for an employee in the data table.
        /// </summary>
        /// <param name="id">the ID of the employee to view the details of</param>
        public static void viewEmployeeDetailsDialog(string id)
        {
            EmployeeDetails details = new EmployeeDetails(id);
            details.ShowDialog();
        }

        /// <summary>
        /// Gets the information of the employee with the specified ID.
        /// </summary>
        /// <param name="id">the ID of the employee to get the details of</param>
        /// <returns>the array of strings with each index representing a property of that employee's information</returns>
        public static string[] getDetails(string id)
        {
            string[] details = new string[9];
            DataRow employeeRow = dt.Rows.Find(id);
            if (employeeRow != null)
            {
                for (int i = 0; i < dataColumnsCount; i++)
                    details[i] = employeeRow[i].ToString();
            }
            return details;
        }

        public void copyFormat(int rowStart, int rowNum)
        {
            FarPoint.Win.Spread.Model.CellRange cr = new FarPoint.Win.Spread.Model.CellRange(0, 0, 1, 8);
            fpSpread1.Sheets[1].GetClipDataObject(false, cr, FarPoint.Win.Spread.ClipboardCopyOptions.Formatting);
            FarPoint.Win.Spread.Model.CellRange cr2 = new FarPoint.Win.Spread.Model.CellRange(rowStart, 0, rowNum, 8);
            fpSpread1.Sheets[0].ClipboardPaste(FarPoint.Win.Spread.ClipboardPasteOptions.Formatting, fpSpread1.Sheets[1].GetClipDataObject(false, cr,
                                                FarPoint.Win.Spread.ClipboardCopyOptions.Formatting), cr2);
        }

        private void fpSpread1_ButtonClicked(object sender, FarPoint.Win.Spread.EditorNotifyEventArgs e)
        {
            fpSpread1.SuspendLayout();
            if (fpSpread1.Sheets[0].ActiveCell.Row.Index == 0 && fpSpread1.Sheets[0].ActiveCell.Column.Index == 6)
            {
                if(searchEmployeeDialog())
                    copyFormat(2, dt.Rows.Count);
            }
            else if (fpSpread1.Sheets[0].ActiveCell.Row.Index == 1 && fpSpread1.Sheets[0].ActiveCell.Column.Index == 6)
            {
                if (addEmployeeForm())
                {
                    copyFormat(fpSpread1.Sheets[0].RowCount - 1, 1);
                }
            }
            else if (fpSpread1.Sheets[0].ActiveCell.Column.Index == 7)
            {
                viewEmployeeDetailsDialog(dt.Rows[fpSpread1.Sheets[0].ActiveCell.Row.Index - 2][0].ToString());
                copyFormat(fpSpread1.Sheets[0].ActiveCell.Row.Index, 1); 
            }
            else if (fpSpread1.Sheets[0].ActiveCell.Column.Index == 6)
            {
                DialogResult dr = MessageBox.Show("Are you sure you want to remove " + fpSpread1.Sheets[0].Cells[fpSpread1.Sheets[0].ActiveCell.Row.Index, 1].Text + "?",
                                                    "Remove Employee",
                                                    MessageBoxButtons.YesNo, MessageBoxIcon.Question);
                if (dr == DialogResult.Yes)
                {
                    fpSpread1.Sheets[0].RemoveRows(fpSpread1.ActiveSheet.ActiveRow.Index, 1);
                }
            }
            fpSpread1.ResumeLayout();
        }

        private void openToolStripMenuItem_Click(object sender, EventArgs e)
        {
            string filename = "";
            OpenFileDialog ofd = new OpenFileDialog();
            ofd.Filter = "XML Files (*.xml)|*.xml|All files (*.*)|*.*";
            DialogResult dr = ofd.ShowDialog();
            if (dr == DialogResult.OK)
            {
                filename = ofd.FileName;
            }
            if (filename != "")
            {
                dt.Clear();
                dt.ReadXml(filename + "DataTable");
                fpSpread1.Open(filename);

                FarPoint.Win.Spread.Data.SpreadDataBindingAdapter data = new FarPoint.Win.Spread.Data.SpreadDataBindingAdapter();
                data.DataSource = dt;
                data.Spread = fpSpread1;
                data.SheetName = "Employees Table";
                data.MapperInfo = new FarPoint.Win.Spread.Data.MapperInfo(2, 0, dt.Rows.Count, 6);
                data.FillSpreadDataByDataSource();

                for (int i = 2; i < dt.Rows.Count + 2; i++)
                {
                    copyFormat(i, 1);
                }
            }
        }

        private void saveToolStripMenuItem_Click(object sender, EventArgs e)
        {
            string filename = "";
            SaveFileDialog sfd = new SaveFileDialog();
            sfd.Filter = "XML Files (*.xml)|*.xml|All files (*.*)|*.*";
            DialogResult dr = sfd.ShowDialog();
            if (dr == DialogResult.OK)
            {
                filename = sfd.FileName;
            }
            if (filename != "")
            {
                fpSpread1.Save(filename, FarPoint.Win.Spread.SaveXMLOptions.All);
                dt.WriteXml(filename + "DataTable");
            }
        }
    }
}
